(in-package "GWB")

(define-condition gwb-error (error)
  ()
  )

(define-condition gwb-simple-error (simple-condition gwb-error)
  ()
  )

(define-condition gwb-syntax-error (gwb-error)
  ((form :initarg :form
         :initform nil
         :reader gwb-sytax-error-form))
  (:report "Syntax error.")
  )

(define-condition gwb-cell-error (gwb-error)
  ((kind :initarg :kind
         :initform "name"
         :reader gwb-cell-error-kind)
   (name :initarg :name
         :initform nil
         :reader gwb-cell-error-name))
  (:report
   (lambda (c s)
     (format s "Unknown ~A~@[ ~A~]."
             (gwb-cell-error-kind c)
             (gwb-cell-error-name c))))
  )

(define-condition gwb-play-error (gwb-error)
  ((strategy :type (or nil strategy)
             :initarg :strategy
             :initform nil
             :reader gwb-play-error-strategy))
  (:report
   (lambda (c s)
     (format s "Playing in ~A strategy"
             (if (gwb-play-error-strategy c)
                 (type-of (gwb-play-error-strategy c))
               "a"))))
  )

(define-condition gwb-move-error (gwb-play-error)
  ((move :type (or nil move)
         :initarg :move
         :initform nil
         :reader gwb-move-error-move))
  (:report
   (lambda (c s)
     (format s "Bad move~@[: `~A'~]."
             (if (gwb-move-error-move c)
                 (move-name (gwb-move-error-move c))
               nil))))
  )

(define-condition gwb-root-move-error (gwb-move-error)
  ()
  (:report
   (lambda (c s)
     (format s "Move ~@[`~A' ~]is at root."
             (if (gwb-move-error-move c)
                 (move-name (gwb-move-error-move c))
               nil))))
  )

(define-condition gwb-move-comp-error (gwb-move-error)
  ((comp :type (or nil vector)
         :initarg :comp
         :initform nil
         :reader gwb-move-comp-error-comp))
  (:report
   (lambda (c s)
     (let ((move-name
            (if (gwb-move-error-move c)
                (move-name (gwb-move-error-move c))
              nil)))
     (if (not (gwb-move-comp-error-comp c))
         (format s "Move ~@[`~A' ~]in wrong component." move-name)
       (format s "Move ~@[`~A' ~]not in component ~A."
               move-name
               (comp-name (gwb-move-comp-error-comp c))))))))
   

(define-condition gwb-play-move-error (gwb-move-error gwb-play-error)
  ()
  (:report
   (lambda (c s)
     (format s "Bad move in a ~A~@[: `~A'~]."
             (if (gwb-play-error-strategy c)
                 (type-of (gwb-play-error-strategy c))
               "strategy")
             (if (gwb-move-error-move c)
                 (move-name (gwb-move-error-move c))
               nil))))
  )

(define-condition gwb-translation-error (gwb-play-error)
  ()
  (:report "Unable to translate a move.")
  )

